;keys: (output line, input read)
;1 = %0111 1111, %1111 1110
;2 = %0111 1111, %1111 0111

org $c000

def cia1	$dc00
def cia2	$dd00
def row		0
def col		1
def cra		14
def crb		15
def icr		%1101
def ddra	2
def ddrb	3
def pra		0
def prb		1

	sei
	lda	#<dumbint	;replace NMI with dumbint
	sta	$fffa
	lda	#>dumbint
	sta	$fffb
	lda	#127		;disable interrupts from CIAs
	sta	cia1+icr
	sta	cia2+icr
	lda	cia1+cra	;stop timers, first CIA
	and	#$fe
	sta	cia1+cra
	lda	cia1+crb
	and	#$fe
	sta	cia1+crb
	lda	cia2+cra	;second CIA
	and	#$fe
	sta	cia2+cra
	lda	cia2+crb
	and	#$fe
	sta	cia2+crb
	lda	#$ff		;force CIA1 into keyboard mode
	sta	cia1+ddra	;and CIA2 for serial comm. over port
	lda	#0
	sta	cia1+ddrb
	sta	cia2+ddrb
	lda	#$3f
	sta	cia2+ddra
	lda	#%11		;clear output reg
	sta	cia2+pra
	lda	1		;remove ROMs and keep IO lines
	ora	#%00000101
	and	#%11111101
	sta	1

	jsr	$c329		;printreset
	lda	#0
	jsr	$c300		;printbin
	jsr	$c339		;printnextline
	lda	#0		;led on
	jsr	sendbyte
	lda	#0
	jsr	$c300		;printbin
	jsr	$c339		;printnextline
	lda	#2		;read memory
	jsr	sendbyte
	lda	#0
	jsr	$c300		;printbin
	jsr	$c339		;printnextline
	lda	#0		;address low
	jsr	sendbyte
	lda	#0
	jsr	$c300		;printbin
	jsr	$c339		;printnextline
	lda	#$c0		;address hi
	jsr	sendbyte
	lda	#0
	jsr	$c300		;printbin
	jsr	$c339		;printnextline
	jsr	receivebyte	;get data from given address
	jsr	$c300		;printbin
	jmp	.		;loop forever

;;on the port, pins:
;; 5 - data
;; 4 - clock
;; 3 - atn
;;serial com. on cia2, port A
;;data in:   10000000
;;clock in:  01000000
;;data out:  00100000
;;clock out: 00010000
;;atn out:   00001000
;;2 lower bits of cia2 should always be 1
;;ledoff: send the LEDOFF command to the floppy
;ledoff:	lda	#%00100011	;send command %01 to the floppy
;	sta	cia2+pra	;send first bit (1), so negate as 0
;	eor	#%00001000
;	sta	cia2+pra
;	tax
;t1:	lda	cia2+pra	;wait for clock in to be 1
;	and	#%01000000
;	bne	t1
;	txa
;	and	#%11011111	;send second bit (0), so negate as 1
;	sta	cia2+pra
;	eor	#%00001000
;	sta	cia2+pra
;t2:	lda	cia2+pra	;wait for clock in to be 0
;	and	#%01000000
;	beq	t2
;	jmp	k1		;done
;
;;ledon: send the LEDON command to the floppy
;ledon:	lda	#%00000011	;send command %00 to the floppy
;	sta	cia2+pra
;	eor	#%00001000
;	sta	cia2+pra	;send first bit (0)
;	tax
;o1:	lda	cia2+pra	;wait for clock in to be 1
;	and	#%01000000
;	bne	o1
;	txa			;send second bit (0)
;	and	#%11011111
;	sta	cia2+pra
;	eor	#%00001000
;	sta	cia2+pra
;o2:	lda	cia2+pra	;wait for clock in to be 0
;	and	#%01000000
;	beq	o2
;	jmp	k1		;done

;sends the byte in A, higher bits first, waits forever, no timeout!
;smokes A, X, Y, flags
sendbyte:
	ldx	#4
sb1:	rol			;send 1st bit
	bcs	sb1_1
	ldy	#%00000011	;bit is 0
	sty	cia2+pra
	ldy	#%00001011
	sty	cia2+pra
	bcc	sb2
sb1_1:	ldy	#%00100011	;bit is 1
	sty	cia2+pra
	ldy	#%00101011
	sty	cia2+pra
sb2:	tay
sb2_2:	lda	cia2+pra	;wait for ack
	and	#%01000000
	bne	sb2_2
	tya
	rol			;send 2nd bit
	bcs	sb3_1
	ldy	#%00001011	;bit is 0
	sty	cia2+pra
	ldy	#%00000011
	sty	cia2+pra
	bcc	sb4
sb3_1:	ldy	#%00101011	;bit is 1
	sty	cia2+pra
	ldy	#%00100011
	sty	cia2+pra
sb4:	tay
sb4_2:	lda	cia2+pra	;wait for ack
	and	#%01000000
	beq	sb4_2
	tya
	dex			;loop 4 times
	bne	sb1
	ldy	#%11		;clear output reg
	sty	cia2+pra
	rts

;waits for a byte, returns it in A, waits for ever, no timeout!
;smokes X, Y, flags
recv: .byte 0
receivebyte:
	ldx	#8
	ldy	#%00001011
rb0:	clc
rb00:	lda	cia2+pra	;wait for data in
	and	#%01000000
rb_:	bne	rb00
	lda	cia2+pra
	sty	cia2+pra
	bmi	rb1		;if we bmi then bit is 1 means we get a 0
	sec			;(it's reversed)
rb1:	lda	recv
	rol
	sta	recv
	lda	rb_		;self-modifying code: beq/bne switch
	eor	#%00100000	;beq is $f0 and bne is $d0
	sta	rb_
	tya
	eor	#%00001000
	tay
	dex
	bne	rb0
	lda	recv
	rts

dumbint:rti
